@@ -13,7 +13,7 @@ pub mod file_locations;
 mod path;
 mod subclass_attrs;
 
-pub use config::{IncludeCppConfig, RustFun, Subclass, UnsafePolicy};
+pub use config::{AllowlistEntry, IncludeCppConfig, RustFun, Subclass, UnsafePolicy};
 use file_locations::FileLocationStrategy;
 pub use path::RustPath;
 use proc_macro2::TokenStream as TokenStream2;
